DEFINE CLASS MakePersonSet AS MakeDataSet OF MakeDatav2.PRG

nSetSize = 1000
cGeneratorClass = "MakePerson"
cGeneratorClassLib = "MakePeoplev2.PRG"

PROCEDURE OpenTables
* Open or create tables

IF FILE("Person")
	USE Person IN 0
	* Grab SS#'s already in use
	SELECT cSSN FROM Person INTO CURSOR __SSNs READWRITE 
	INDEX on cSSN TAG cSSN
ELSE
	CREATE TABLE Person (iID I AUTOINC UNIQUE, cFirst C(15), cLast C(30), ;
                          cGender C(1), cSSN C(9), dBirth D)
ENDIF

IF FILE("Address")
	USE Address IN 0
ELSE 
	CREATE TABLE Address (iID I AUTOINC UNIQUE, iPersonFK I, iLocFK I, cStreet c(60), cCity C(20), cState C(2), cZip C(9))
ENDIF

IF FILE("Phone")
	USE Phone IN 0
ELSE 
	CREATE Table Phone (iID I AUTOINC UNIQUE, iPersonFK I, iLocFK I, cType C(8), cNumber C(10))
ENDIF 

IF FILE("Email")
	USE Email IN 0
ELSE 
	CREATE TABLE Email (iID I AUTOINC UNIQUE, iPersonFK I, iLocFK I, mEmail M)
ENDIF 

IF FILE("URL")
	USE URL IN 0
ELSE 
	CREATE TABLE URL (iID I AUTOINC UNIQUE, iPersonFK I, iLocFK I, mURL M)
ENDIF 

IF FILE("Location")
	USE Location IN 0
ELSE
	CREATE TABLE Location (iID I AUTOINC UNIQUE, cLocation C(8))
	INDEX on UPPER(cLocation) TAG cLocation
ENDIF

RETURN
ENDPROC

PROCEDURE SetData

This.AddDataType("Person", This.nSetSize)

RETURN
ENDPROC

PROCEDURE SaveRecord(oRecord)

LOCAL iPerson, iLoc

WITH oRecord
	INSERT INTO Person (cFirst, cLast, cGender, cSSN, dBirth) ;
		VALUES (.cFirst, .cLast, .cGender, .cSSN, .dBirthdate)
	iPerson = Person.iID

	FOR EACH oAddress IN .oAddresses FOXOBJECT
		WITH oAddress
			iLoc = This.CheckLookup(.cType, "Location", "cLocation", "cLocation", "iID")
			INSERT INTO Address (iPersonFK, iLocFK, cStreet, cCity, cState, cZip) ;
				VALUES (m.iPerson, m.iLoc, .Street, .City, ;
				        .State, .Zip)
		ENDWITH 
	ENDFOR  
	
	FOR EACH oPhone IN .oPhones FOXOBJECT
		WITH oPhone
			iLoc = This.CheckLookup(.cLoc, "Location", "cLocation", "cLocation", "iID")
			INSERT INTO Phone (iPersonFK, iLocFK, cType, cNumber) ;
				VALUES (m.iPerson, m.iLoc, .cType, ALLTRIM(.AreaCode) + ALLTRIM(.Number))
		ENDWITH
	ENDFOR
	
	FOR EACH oEmail IN .oEmails FOXOBJECT
		WITH oEmail
			iLoc = This.CheckLookup(.cType, "Location", "cLocation", "cLocation", "iID")
			INSERT INTO Email (iPersonFK, iLocFK, mEmail) ;
				VALUES (m.iPerson, m.iLoc, .Email)
		ENDWITH 
	ENDFOR 
	
	FOR EACH oURL IN .oWeb FOXOBJECT
		WITH oURL
			iLoc = This.CheckLookup(.cType, "Location", "cLocation", "cLocation", "iID")
			INSERT INTO URL (iPersonFK, iLocFK, mURL) ;
				VALUES (m.iPerson, m.iLoc, .URL)
		ENDWITH
	ENDFOR 
	
ENDWITH

RETURN
ENDPROC

PROCEDURE About
*==============================================================================
* Class:				MAKEPERSONSET
* Based On: 			MAKEDATASET
* Purpose:				Generate a set of test data for person data
* Author:				Tamar Granor
* Copyright:			(c) 2005 Tamar E. Granor, Ph.D.
* Last revision:		08/09/05
* Include file:			None

* Changes in "Based On" class properties:
*	cGeneratorClass = "MakePerson"
*	cGeneratorClassLib = "MakePeople.PRG"
*	nSetSize = 5000
*
* Changes in "Based On" class methods:
*	OpenTables - creates or opens tables to store person data
*	SaveRecord - saves person data to tables
*
* Custom public properties added:
*	None
*
* Custom protected properties added:
*	None
*
* Custom public methods added:
*	None
*
* Custom protected methods added:
*	None
*	
*==============================================================================
RETURN
ENDPROC

ENDDEFINE


DEFINE CLASS MakePerson AS MakeRecord OF MakeDatav2.PRG
* Abstract class containing code for typical person
* data scenarios.

* Ranges
** Birthdates
dOldest = {^ 1900-1-1}
dYoungest = DATE()
nDates = 0 && Number of dates in the date range
nDomainWordMax = 3 && Maximum number of combined words for domain name

** Addresses
nLowHouseDigits = 1 && # of digits
nHighHouseDigits = 5 && * of digits


* Probabilities
nMale = .5 && Determines what percent of result records are male
nHasExtension = 0 && Determines what percent of phone numbers have extensions

** Addresses
DIMENSION aAddresses[1,2] && probabilities for different address types
nHasLetter = .3 && is there a letter after the number
DIMENSION aPhones[1,3] && probabilies for different phone types
DIMENSION aEmails[1,2] && probabilities for different emails
DIMENSION aWeb[1,2] && probabilities for different websites

PROCEDURE SetData

WITH This
	.AddData("RawData\LastNames", "LastNames")
	.AddData("RawData\BoysNames", "BoysNames")
	.AddData("RawData\GirlsNames", "GirlsNames")
	.AddData("RawData\StreetNames", "Streets")
	.AddData("RawData\Cities", "Cities")
	.AddData("RawData\AreaCode", "AreaCode")
	.AddData("RawData\Domains", "Domains")
	.AddData("RawData\TLDs", "TLDs")
ENDWITH

This.nDates = This.dYoungest - This.dOldest + 1

RETURN
ENDPROC

PROCEDURE SetProbabilities

WITH This
	DIMENSION .aAddresses[3,2]
	.aAddresses[1,1] = "Personal"
	.aAddresses[1,2] = .9
	.aAddresses[2,1] = "Business"
	.aAddresses[2,2] = .4
	.aAddresses[3,1] = "School"
	.aAddresses[3,2] = .2

	DIMENSION .aPhones[8,3]
	.aPhones[1,1] = "Personal"
	.aPhones[1,2] = "Voice"
	.aPhones[1,3] = .9
	.aPhones[2,1] = "Personal"
	.aPhones[2,2] = "Fax"
	.aPhones[2,3] = .3
	.aPhones[3,1] = "Business"
	.aPhones[3,2] = "Voice"
	.aPhones[3,3] = .6
	.aPhones[4,1] = "Business"
	.aPhones[4,2] = "Fax"
	.aPhones[4,3] = .4
	.aPhones[5,1] = "School"
	.aPhones[5,2] = "Voice"
	.aPhones[5,3] = .2
	.aPhones[6,1] = "School"
	.aPhones[6,2] = "Fax"
	.aPhones[6,3] = .1
	.aPhones[7,1] = "Personal"
	.aPhones[7,2] = "Cell"
	.aPhones[7,3] = .75
	.aPhones[8,1] = "Personal"
	.aPhones[8,2] = "Pager"
	.aPhones[8,3] = .2

	DIMENSION .aEmails[2,2]
	.aEmails[1,1] = "Personal"
	.aEmails[1,2] = .7
	.aEmails[2,1] = "Business"
	.aEmails[2,2] = .8
	
	DIMENSION .aWeb[2,2]
	.aWeb[1,1] = "Personal"
	.aWeb[1,2] = .2
	.aWeb[2,1] = "Business"
	.aWeb[2,2] = .6

ENDWITH

RETURN
ENDPROC

PROTECTED PROCEDURE SetMethods

WITH This
	.AddMethod("GetName","Person")
	.AddMethod("GetBirthdate","Person")
	.AddMethod("GetAddresses","Person")
	.AddMethod("GetPhones","Person")
	.AddMethod("GetEmails","Person")
	.AddMethod("GetURLs","Person")
	.AddMethod("GetSSN","Person")
ENDWITH

RETURN
ENDPROC

PROTECTED PROCEDURE GetName
* Get all components of the name

LOCAL nRec, nRand

* Choose a last name
ADDPROPERTY(This.oRecord, "cLast", ALLTRIM(This.RandRecord("LastNames","cName")))

* Determine male or female and get first name
nRand = RAND()
IF nRand <= This.nMale
	ADDPROPERTY(This.oRecord, "cFirst", ALLTRIM(This.RandRecord("BoysNames","cName")))
	ADDPROPERTY(This.oRecord, "cGender", "M")
ELSE
	ADDPROPERTY(This.oRecord, "cFirst", ALLTRIM(This.RandRecord("GirlsNames","cName")))
	ADDPROPERTY(This.oRecord, "cGender", "F")
ENDIF

RETURN
ENDPROC 

PROTECTED PROCEDURE GetBirthdate
* Generate a birthdate for this person
* Note that this code is unrealistic as it spreads
* the birthdates evenly throughout the range rather than
* weighting based on age frequencies. For most situations,
* this is sufficient.

LOCAL nRand

nRand = This.RandInt(1, This.nDates)
ADDPROPERTY(This.oRecord, "dBirthdate", This.dOldest + nRand - 1)

RETURN
ENDPROC

PROTECTED PROCEDURE GetAddresses
* Generate addresses. Use the array aAddresses to determine
* which addresses to generate .
* Store addresses in the record's oAddresses collection

LOCAL nAddr, nRand, oAddress
LOCAL nHouseNumber, cHouseLetter, nHigh, nLow

ADDPROPERTY(This.oRecord, "oAddresses", CREATEOBJECT("Collection"))

FOR nAddr = 1 TO ALEN(This.aAddresses, 1)
	nRand = RAND()
	IF nRand <= This.aAddresses[ m.nAddr, 2]
		* Generate this one
		oAddress = CREATEOBJECT("Empty")
		ADDPROPERTY(oAddress,"cType", This.aAddresses[m.nAddr, 1])

		* Get a house number. First, figure out how many digits, 
		* then choose a random value with that many digits.
		* This approach is used because choosing randomly over 
		* the whole range results in too many longer values.
		nRand = This.RandInt(This.nLowHouseDigits, This.nHighHouseDigits)
		nLow = 10^(nRand-1)
		nHigh = 10^nRand - 1 
		nHouseNumber = This.RandInt(m.nLow, m.nHigh)
		* Check whether to add a letter
		nRand = RAND()
		IF nRand <= This.nHasLetter
			cHouseLetter = This.RandLetter()
		ELSE
			cHouseLetter = ""
		ENDIF
		cHouseNumber = TRANSFORM(m.nHouseNumber) + m.cHouseLetter
		
		* Get a street
		* Use method to move to correct record, but need to
		* retrieve multiple fields
		This.RandRecord("Streets","cStreet")
		cStreet = Streets.cDir -(" " + Streets.cStreet) - (" " + Streets.cType)
		
		* Get a city, state, zip combination
		This.RandRecord("Cities","cCity")

		ADDPROPERTY(oAddress,"Street", m.cHouseNumber + " " + ALLTRIM(m.cStreet))
		ADDPROPERTY(oAddress,"City", Cities.cCity)
		ADDPROPERTY(oAddress,"State", Cities.cState)
		ADDPROPERTY(oAddress,"Zip", Cities.cZip)
		ADDPROPERTY(This.oRecord, "AreaCode", Cities.cACode)
		
		* Now add the new address to the collection
		This.oRecord.oAddresses.Add(m.oAddress)
	ENDIF 
ENDFOR

RETURN
ENDPROC

PROTECTED PROCEDURE GetPhones
* Generate phone numbers. Use the array aPhones to determine
* which phone numbers to generate for this person.
* Store phone numbers in the Person's oPhones collection.

LOCAL nPhone, nRand, oPhone

ADDPROPERTY(This.oRecord, "oPhones", CREATEOBJECT("Collection"))

FOR nPhone = 1 TO ALEN(This.aPhones, 1)
	nRand = RAND()
	IF nRand <= This.aPhones[ m.nPhone, 3]
		* Generate this one
		oPhone = CREATEOBJECT("Empty")
		ADDPROPERTY(oPhone,"cLoc", This.aPhones[nPhone, 1])
		ADDPROPERTY(oPhone,"cType", This.aPhones[nPhone, 2])

		* Create a phone number. Use the area code already saved,
		* if there is one.
		
		IF PEMSTATUS(This.oRecord, "AreaCode", 5)
			cAreaCode = This.oRecord.AreaCode
		ELSE 
			cAreaCode = This.RandRecord("AreaCode","NPA")
		ENDIF
		
		nFirstDigit = This.RandInt(2, 9)
		cNumber = TRANSFORM(m.nFirstDigit)
		* Loop to get rest of digits.
		FOR nDigit = 2 TO 7
			cNumber = m.cNumber + TRANSFORM(This.RandInt(0,9))
		ENDFOR
		
		ADDPROPERTY(oPhone,"AreaCode", m.cAreaCode)
		ADDPROPERTY(oPhone,"Number", m.cNumber)
		
		nRand = RAND()
		IF nRand <= This.nHasExtension
			nRand = This.RandInt(2, 4) && number of digits
			nLow = INT(10^(nRand-1))
			nHigh = INT(10^nRand-1)
			nRand = This.RandInt(nLow, nHigh)
			cExt = PADL(nRand, 4, "0")
		ELSE
			cExt = ""
		ENDIF 
		ADDPROPERTY(oPhone, "Extension", m.cExt)
		
		* Now add the new phone to the collection
		This.oRecord.oPhones.Add(m.oPhone)
	ENDIF 
ENDFOR

RETURN
ENDPROC

PROTECTED PROCEDURE GetEmails
* Generate email addresses. Use the array aEmails to determine
* which email addresses to generate for this person.
* Store email addresses in the Person's oEmails collection.

LOCAL nEmail, nRand, oEmail

ADDPROPERTY(This.oRecord, "oEmails", CREATEOBJECT("Collection"))

FOR nEmail = 1 TO ALEN(This.aEmails, 1)
	nRand = RAND()
	IF nRand <= This.aEmails[ nEmail, 2]
		* Generate this one
		oEmail = CREATEOBJECT("Empty")
		ADDPROPERTY(oEmail,"cType", This.aEmails[nEmail, 1])

		* Create a Email address. Do the user name, domain and TLD
		* separately.
		
		* For user name, split 1/3 each:
		*	- first name only
		*	- first initial + last name
		*	- first name + last name
		nRand = This.RandInt(1,3)
		DO CASE 
		CASE nRand = 1
			* First name only
			cUserName = This.oRecord.cFirst
		CASE nRand = 2
			* First initial + last name
			cUserName = LEFT(This.oRecord.cFirst, 1) + ;
			            This.oRecord.cLast
		CASE nRand = 3
			* First name + last name
			cUserName = This.oRecord.cFirst + This.oRecord.cLast
		ENDCASE
		
		* For domain, generate from word list
		cDomain = This.GetDomainName()
		
		* Pick a TLD
		cDomain = m.cDomain - "." - This.GetTLD()

		ADDPROPERTY(oEmail,"Email", ALLTRIM(m.cUserName) + "@" + ALLTRIM(m.cDomain))
	
		* Now add the new email to the collection
		This.oRecord.oEmails.Add(m.oEmail)
	ENDIF 
ENDFOR

RETURN
ENDPROC

PROTECTED PROCEDURE GetDomainName
* Generate a random domain name

LOCAL nRand, nDomain, cDomain

* First, figure our how many words
nRand = This.RandInt(1, This.nDomainWordMax)

* Now get that many words
cDomain = ""
FOR nDomain = 1 TO nRand
	cDomain = cDomain - This.RandRecord("Domains","cWord")
ENDFOR

RETURN cDomain
ENDPROC 

PROTECTED PROCEDURE GetTLD
* Generate a random TLD

LOCAL nRand, cTLD

cTLD = This.RandRecord("TLDs","TLD")

RETURN cTLD
ENDPROC

PROTECTED PROCEDURE GetURLs
* Generate URLs. Use the array aWeb to determine
* which URLs to generate for this person.
* Store URLs in the Person's oWeb collection.

LOCAL nURL, cURL

ADDPROPERTY(This.oRecord, "oWeb", CREATEOBJECT("Collection"))

FOR nURL = 1 TO ALEN(This.aWeb, 1)
	nRand = RAND()
	IF nRand <= This.aWeb[ m.nURL, 2]
		oURL = CREATEOBJECT("Empty")
		ADDPROPERTY(oURL, "cType", This.aWeb[ m.nURL, 1])
		
		* Create a web address
		cURL = "www" + "." + This.GetDomainName()
		cURL = ALLTRIM(m.cURL - "." - This.GetTLD())
		
		* Add it
		ADDPROPERTY(oURL, "URL", m.cURL)
		
		This.oRecord.oWeb.Add(m.oURL)
	ENDIF

ENDFOR

RETURN
ENDPROC

PROTECTED PROCEDURE GetSSN
* Generate a random social security number
* Digits in use as of 7-2005.

LOCAL cSSN, nDigit1, nDigit2, nDigit3, nMiddle, nLast, lNewNum

IF NOT USED("__SSNs")
	CREATE CURSOR __SSNs (cSSN C(9))
	INDEX on cSSN TAG cSSN
ENDIF

lNewNum = .F.
DO WHILE NOT lNewNum
	* First set of three: 001 to 772
	nDigit1 = This.RandInt(0, 7) && First digit not above 7
	IF m.nDigit1 = 7
		nDigit2 = This.RandInt(0, 7) 
		IF m.nDigit2 = 7
			nDigit3 = This.RandInt(0, 2)
		ELSE
			nDigit3 = This.RandInt(0, 9)
		ENDIF 
	ELSE
		nDigit2 = This.RandInt(0, 9)
		IF m.nDigit1 = 0 AND nDigit2 = 0
			nDigit3 = This.RandInt(1, 9)
		ELSE 
			nDigit3 = This.RandInt(0, 9)
		ENDIF 
	ENDIF

	cSSN = TRANSFORM(m.nDigit1) + TRANSFORM(m.nDigit2) + TRANSFORM(m.nDigit3)

	* Second set of two: 01 to 99
	nMiddle= This.RandInt(1, 99)
	cSSN = m.cSSN + PADL(m.nMiddle,2,"0")

	* Third set of four: 0001 to 9999
	nLast = This.RandInt(1, 9999)
	cSSN = m.cSSN + PADL(m.nLast, 4, "0")

	* Is it unique?
	IF NOT SEEK(m.cSSN, "__SSNs", "cSSN")
		lNewNum = .T.
		INSERT INTO __SSNs VALUES (m.cSSN)
	ENDIF
ENDDO

ADDPROPERTY(This.oRecord, "cSSN", m.cSSN)

RETURN
ENDPROC

PROCEDURE About
*==============================================================================
* Class:				MAKEPERSON
* Based On: 			MakeData
* Purpose:				Generate test data for a person
* Author:				Tamar Granor
* Copyright:			(c) 2005 Tamar E. Granor, Ph.D.
* Last revision:		08/09/05
* Include file:			None

* Changes in "Based On" class properties:
*	None
*
* Changes in "Based On" class methods:
*	SetData - opens raw data tables for person generation
*	SetMethods - sets up order of method calls 
*	SetProbabilities - fills in probability arrays
*
* Custom public properties added:
*	aAddresses[1,2] - probability person has different address types
*	aEmails[1,2] - probability person has different email types
*	aPhones[1,3] - probability person has different phone types
*	aWeb[1,2] - probability person has for different website types
*	dOldest - first birthdate to use
*	dYoungest - last birthdate to use
*	nDates - number of days in the date range
*	nDomainWordMax - maximum number of combined words for domain name
*	nHasLetter - probability of a letter after the number in street address
*	nHighHouseDigits - most digits in street address
*	nLowHouseDigits - fewest digits in street address
*	nMale - what percent of result records are male
*
* Custom protected properties added:
*	None
*
* Custom public methods added:
*	None
*
* Custom protected methods added:
*	GetAddresses - generates addresses
*	GetBirthdate - generates birthdate
*	GetDomainName - generates a domain name
*	GetEmails - generates email addresses
*	GetName - generates first name, last name and gender
*	GetPhones - generates phone numbers
*	GetSSN - generates a social security number
*	GetTLD - generates a top-level domain
*	GetURLs - generates web addresses
*	
*==============================================================================
RETURN
ENDPROC

ENDDEFINE

