DEFINE CLASS MakeSchoolSet AS MakePersonSet OF MakePeoplev2.PRG

cGeneratorClass = "MakeSchoolData"
cGeneratorClassLib = "MakeSchoolDatav2.PRG"

PROCEDURE OpenTables

OPEN DATABASE School
USE Classes IN 0
USE Departments IN 0
USE Students IN 0
USE Instructors IN 0
USE Students_And_Classes IN 0

RETURN
ENDPROC
	
PROCEDURE CloseTables

SET DATABASE TO School
CLOSE DATABASES

RETURN
ENDPROC

PROCEDURE SetData
* Set up the list or record types and counts to generate

This.AddDataType("Department", 7)
This.AddDataType("Class", 1000)
This.AddDataType("Instructor", 1000)
This.AddDataType("Student", 1000)

RETURN
ENDPROC

PROCEDURE SaveRecord(oRecord, cType)
* Save data into the appropriate table(s)

WITH oRecord
	DO CASE 
	CASE cType = "Department"
		INSERT INTO Departments ;
			(DepartmentName, DepartmentNumber, DepartmentManager, DepartmentChairPerson) ;
			VALUES ;
			(.Department, .DeptNum, .DeptMgr, .DeptChair)
	
	CASE cType = "Class"
		INSERT INTO Classes ;
			(ClassName, SectionNumber, Term, Units, Year, DaysAndTimes) ;
			VALUES ;
			(.CourseName, .Section, .Term, .Units, .Year, .Meetings)
			
	CASE cType = "Instructor"
		INSERT INTO Instructors ;
			(Instructor, EmailName, PhoneNumber, Extension) ;
			VALUES ;
			(.cFirst + " " + .cLast, .oEmails[1].Email, ;
			 .oPhones[1].AreaCode + .oPhones[1].Number, .oPhones[1].Extension)
			
	CASE cType = "Student"
		INSERT INTO Students ;
			(FirstName, LastName, ParentsNames, Address, ;
			 City, StateOrProvince, PostalCode, PhoneNumber, ;
			 EmailName, StudentNumber) ;
			VALUES ;
			(.cFirst, .cLast, .ParentsNames, .oAddresses[1].Street, ;
			 .oAddresses[1].City, .oAddresses[1].State, .oAddresses[1].Zip, ;
			 .oPhones[1].AreaCode + .oPhones[1].Number, ;
			 .oEmails[1].Email, .cStudentNumber)

	ENDCASE 
ENDWITH
	
RETURN
ENDPROC

PROCEDURE AfterMakeSet
* Add linking data

* First, link courses to departments and instructors
SELECT Classes
SCAN 
	REPLACE DepartmentID WITH This.GetRandRecord("Departments", "DepartmentId"), ;
			InstructorID WITH This.GetRandRecord("Instructors", "InstructorID")
ENDSCAN 

* Add major to student
SELECT Students
SCAN
	REPLACE Major WITH This.GetRandRecord("Departments", "DepartmentName")
ENDSCAN

* Add student/course links
FOR nRecord = 1 TO 10000
	nStudent = This.GetRandRecord("Students", "StudentID")
	nCourse = This.GetRandRecord("Classes", "ClassID")
	INSERT INTO Students_And_Classes ;
		(ClassId, StudentID) ;
		VALUES ;
		(m.nCourse, m.nStudent)
ENDFOR 

RETURN
ENDPROC

ENDDEFINE

DEFINE CLASS MakeSchoolData AS MakePerson OF MakePeoplev2.PRG

nSingleParent = .5
nFemaleParent = .7
nMultiSection = .5
nMaxSections = 10
nHasExtension = .5

PROCEDURE SetData

DODEFAULT()
This.AddData("Fields", "Fields")
This.AddData("CoursePrefix", "CoursePrefix")
This.AddData("CourseTopic", "CourseTopic")
This.AddData("Terms", "Terms")
This.AddData("ClassDays", "ClassDays")

RETURN
ENDPROC

PROCEDURE SetProbabilities

WITH This
	DIMENSION .aAddresses[1,2]
	.aAddresses[1,1] = "Personal"
	.aAddresses[1,2] = 1

	DIMENSION .aPhones[1,3]
	.aPhones[1,1] = "Personal"
	.aPhones[1,2] = "Voice"
	.aPhones[1,3] = 1

	DIMENSION .aEmails[1,2]
	.aEmails[1,1] = "Personal"
	.aEmails[1,2] = 1
ENDWITH

RETURN
ENDPROC

PROCEDURE SetMethods

WITH This
	.AddMethod("GetName", "Student")
	.AddMethod("GetAddresses", "Student")
	.AddMethod("GetParents", "Student")
	.AddMethod("GetPhones", "Student")
	.AddMethod("GetEmails", "Student")
	.AddMethod("GetStudentNumber", "Student")
	
	.AddMethod("GetName", "Instructor")
	.AddMethod("GetEmails", "Instructor")
	.AddMethod("GetPhones", "Instructor")
	
	.AddMethod("GetDept", "Department")
	.AddMethod("GetMgr", "Department")
	.AddMethod("GetChair", "Department")
	
	.AddMethod("GetCourse", "Class")
	.AddMethod("GetTerm", "Class")
	.AddMethod("GetUnits", "Class")
	.AddMethod("GetMeetings", "Class")
	
ENDWITH
RETURN
ENDPROC

PROCEDURE GetParents
* Generate names for student's parents

LOCAL nRec, nRand, cName

* Decide whether married or single
nRand = RAND()
IF nRand <= This.nSingleParent
	* Single parent--male or female
	nRand = RAND()
	IF nRand <= This.nFemaleParent
		cName = This.GetRandomName("GirlsNames")
	ELSE
		cName = This.GetRandomName("BoysNames")
	ENDIF
ELSE
	cName = This.GetRandomName("BoysNames")
	cName = m.cName + " and " + This.GetRandomName("GirlsNames")
ENDIF

* Choose a last name
cName = m.cName + " " + This.GetRandomName("LastNames")

ADDPROPERTY(This.oRecord, "ParentsNames", m.cName)

RETURN 

PROCEDURE GetRandomName(cAlias)

LOCAL cName

cName = ALLTRIM(This.RandRecord(cAlias,"cName"))

RETURN m.cName
ENDPROC 

PROCEDURE GetStudentNumber
* Generate a random 8-digit student number

LOCAL nNumber, cNumber, lNewNum

IF NOT USED("__StudNums")
	CREATE CURSOR __StudNums (cNumber C(8))
	INDEX on cNumber TAG cNumber
ENDIF

lNewNum = .F.
DO WHILE NOT lNewNum
	nNumber = This.RandInt(10000000, 99999999)
	cNumber = TRANSFORM(nNumber)

	* Check whether it exists already
	IF NOT SEEK(m.cNumber, "__StudNums", "cNumber")
		lNewNum = .T.
		INSERT INTO __StudNums VALUES (m.cNumber)
	ENDIF
ENDDO

ADDPROPERTY(This.oRecord, "cStudentNumber", cNumber)
RETURN
ENDPROC

PROCEDURE GetDept

LOCAL nRec, lNewName, cName, nAttempts

IF NOT USED("__Depts")
	CREATE CURSOR __Depts (cDept C(50))
	INDEX on cDept TAG cDept
ENDIF

IF RECCOUNT("__Depts") < This.GetDataCount("Fields")
	m.lNewName = .F.
	m.cName = ""
	DO WHILE NOT lNewName 
		cName = This.RandRecord("Fields", "cFieldName")
		
		* Check whether it exists already
		IF NOT SEEK(m.cName, "__Depts", "cDept")
			lNewName = .T.
			INSERT INTO __Depts VALUES (m.cName)
		ENDIF
	ENDDO 

	ADDPROPERTY(This.oRecord, "Department", m.cName)
	ADDPROPERTY(This.oRecord, "DeptNum", This.RandInt(1, 2000))
ENDIF

RETURN
ENDPROC

PROCEDURE GetMgr

LOCAL cMgr

cMgr = This.GetPersonName()

ADDPROPERTY(This.oRecord, "DeptMgr", m.cMgr)

RETURN
ENDPROC

PROCEDURE GetChair

LOCAL cChair

cChair = This.GetPersonName()

ADDPROPERTY(This.oRecord, "DeptChair", m.cChair)

RETURN
ENDPROC

PROCEDURE GetPersonName

LOCAL cName, nRand

nRand = RAND()

IF nRand <=This.nMale
	cName = This.GetRandomName("BoysNames")
ELSE
	cName = This.GetRandomName("GirlsNames")
ENDIF

cName = m.cName + " " + This.GetRandomName("LastNames")

RETURN m.cName
ENDPROC

PROCEDURE GetCourse

LOCAL nRec, cCourse

cCourse = ALLTRIM(This.RandRecord("CoursePrefix","cPrefix"))

cCourse = m.cCourse + " " + ALLTRIM(This.RandRecord("CourseTopic","cTopic"))

ADDPROPERTY(This.oRecord, "CourseName", m.cCourse)

* Now section number
nRand = RAND()
IF nRand <= This.nMultiSection
	nSection = This.RandInt(1, This.nMaxSections)
ELSE
	nSection = 0
ENDIF

ADDPROPERTY(This.oRecord, "Section", m.nSection)

RETURN
ENDPROC

PROCEDURE GetTerm

LOCAL nRec

ADDPROPERTY(This.oRecord, "Term", This.RandRecord("Terms","cTerm"))
ADDPROPERTY(This.oRecord, "Year", This.RandInt(2000, 2005))

RETURN
ENDPROC

PROCEDURE GetUnits

ADDPROPERTY(This.oRecord, "Units", TRANSFORM(This.RandInt(1, 3)))

RETURN
ENDPROC

PROCEDURE GetMeetings

LOCAL nRec

ADDPROPERTY(This.oRecord, "Meetings", This.RandRecord("ClassDays","cDays"))
RETURN
ENDPROC

ENDDEFINE
