DEFINE CLASS MakeDataSet AS Session 

oRecordGenerator = .null.
oRand = .null.
cGeneratorClass = ""
cGeneratorClassLib = ""

oDataToGenerate = .null.

PROCEDURE Init

This.oRand = NEWOBJECT("RandFunctions","RandFuncs.PRG")
This.oRecordGenerator = NEWOBJECT(This.cGeneratorClass, This.cGeneratorClassLib)
This.oDataToGenerate = CREATEOBJECT("Collection")
This.OpenTables()

This.SetData()

RETURN
ENDPROC

PROCEDURE Destroy

This.CloseTables()

ENDPROC


PROCEDURE OpenTables
* Abstract
RETURN
ENDPROC

PROCEDURE CloseTables
* Abstract
RETURN
ENDPROC

PROCEDURE SetData
* Abstract
RETURN
ENDPROC

PROCEDURE AddDataType(cName, nCount)
LOCAL oDataObject

* Make sure the collection exists
IF VARTYPE(This.oDataToGenerate) <> "O"
	This.oDataToGenerate = CREATEOBJECT("Collection")
ENDIF

* Create the data object
oDataObject = CREATEOBJECT("Empty")
ADDPROPERTY(oDataObject, "Type", m.cName)
ADDPROPERTY(oDataObject, "Count", m.nCount)

* Add the object to the collection, using the type as the key
This.oDataToGenerate.Add(oDataObject, m.cName)

RETURN 
ENDPROC 

PROCEDURE MakeSet
* Generate a set of records, storing them
* to tables

LOCAL oDataType, nRecord, oRecord

FOR EACH oDataType IN This.oDataToGenerate FOXOBJECT 
	FOR nRecord = 1 TO oDataType.Count
		oRecord = This.oRecordGenerator.GenerateRecord(oDataType.Type)
		This.SaveRecord(oRecord, oDataType.Type)
	ENDFOR
	
	This.AfterMakeType()
ENDFOR

This.AfterMakeSet()

RETURN
ENDPROC

PROCEDURE SaveRecord( oRecord, cType )
* Abstract

RETURN
ENDPROC

PROCEDURE CheckLookup(cValue, cTable, cKey, cField, cPKField)
* Check whether a specified value is already 
* in a given table. If so, return the PK.
* If not, insert it and return the PK.

LOCAL uReturn, cReturnField

IF NOT SEEK(UPPER(cValue), cTable, cKey)
	INSERT INTO (cTable) (&cField) ;
		VALUES (cValue)
ENDIF

cReturnField = cTable + "." + cPKField
uReturn = EVALUATE(cReturnField)

RETURN uReturn
ENDPROC 

PROCEDURE AfterMakeType
* Abstract for processing after creating data of one type

RETURN
ENDPROC

PROCEDURE AfterMakeSet
* Abstract for processing after creating data set

RETURN
ENDPROC

PROCEDURE GetRandRecord(cAlias, cField)
* Choose a record at random from an existing table and
* return the value of the specified field

*!*	LOCAL nRecord, uReturn, nOldRecNo

*!*	TRY
*!*		nRecord = This.oRand.RandInt(1, RECCOUNT(m.cAlias))
*!*		GO (m.nRecord) IN (m.cAlias)
*!*		STORE EVALUATE(m.cAlias + "." + m.cField) TO uReturn
*!*	CATCH
*!*		uReturn = .null.
*!*	ENDTRY

uReturn = This.oRand.RandRecord(m.cAlias, m.cField)

RETURN uReturn
	

PROCEDURE About
*==============================================================================
* Class:				MAKEDATASET
* Based On: 			Session
* Purpose:				Abstract class for creating test data sets
* Author:				Tamar Granor
* Copyright:			(c) 2005 Tamar E. Granor, Ph.D.
* Last revision:		08/09/05
* Include file:			None

* Changes in "Based On" class properties:
*	None
*
* Changes in "Based On" class methods:
*	Destroy - clean-up
*	Init - initial set-up
*
* Custom public properties added:
*	cGeneratorClass - name of record generator class
*	cGeneratorClassLib - class library for record generator class
*	nSetSize - number of records to generate
*	oRecordGenerator - object reference to record generator class
*
* Custom protected properties added:
*	None
*
* Custom public methods added:
*	About - this documentation method
*	AfterMakeSet - perform any actions needed after creating data set. Abstract.
*	CheckLookup - look for a value in a look-up table. If it's not found,
*                 add it. Either way, return a specified field.
*	CloseTables - close tables opened by this class. Abstract.
*	MakeSet - driver method. Creates and saves records.
*	OpenTables - open tables needed by this class. Abstract.
*	SaveRecord - save the newly-created record. Abstract.
*
* Custom protected methods added:
*	None
*	
*==============================================================================


ENDDEFINE


DEFINE CLASS MakeRecord AS Custom 

oRecord = .null. && holds generated record object
oData = .null. && holds info about data sets
oMethods = .null. && holds infor about methods to call

oRand = .null.

PROCEDURE Init

* Load rand functions
This.oRand = NEWOBJECT("RandFunctions","RandFuncs.PRG")

This.oData = CREATEOBJECT("Collection")
This.oMethods = CREATEOBJECT("Collection")

This.SetProbabilities()
This.SetMethods()
This.SetData()
This.OpenData()

ENDPROC

PROCEDURE Destroy

This.CloseData()
This.oRecord = .null.

ENDPROC

PROCEDURE RandInt (nMin as Integer, ;
                   nMax as Integer) as Integer
* Return a random integer between 0 and the specified max

*!*	LOCAL nRand, nResult

*!*	IF VARTYPE(nMin)<>"N"
*!*		nMin = 0
*!*	ENDIF

*!*	IF VARTYPE(nMax)<> "N"
*!*		nMax = 1
*!*	ENDIF 

*!*	nRand = RAND()
*!*	nResult = INT((nMax - nMin + 1) * nRand) + nMin

nResult = This.oRand.RandInt(m.nMin, m.nMax)

RETURN nResult
ENDPROC

PROTECTED PROCEDURE RandLetter
* Return a randomly selected letter of the alphabet

*!*	LOCAL nRand, cLetter

*!*	nRand = This.RandInt(1, 26)
*!*	cLetter = CHR(64 + nRand)

cLetter = This.oRand.RandLetter()

RETURN cLetter
ENDPROC

PROCEDURE RandRecord(cAlias, cField)
* Choose a record at random from an existing table and
* return the value of the specified field

*!*	LOCAL nRecord, uReturn, nOldRecNo

*!*	TRY
*!*		nRecord = This.oRand.RandInt(1, This.GetDataCount(m.cAlias))
*!*		GO (m.nRecord) IN (m.cAlias)
*!*		STORE EVALUATE(m.cAlias + "." + m.cField) TO uReturn
*!*	CATCH
*!*		uReturn = .null.
*!*	ENDTRY

uReturn = This.oRand.RandRecord(m.cAlias, m.cField)

RETURN uReturn

PROCEDURE SetData
* Specify the tables used in the generation process
* by listing them in an array.
* Abstract

RETURN
ENDPROC

PROCEDURE AddData(cTable, cAlias)
* Add a table to the oData collection

LOCAL oDataObject

* Make sure the collection exists
IF VARTYPE(This.oData) <> "O"
	This.oData = CREATEOBJECT("Collection")
ENDIF

* Create the data object
oDataObject = CREATEOBJECT("Empty")
ADDPROPERTY(oDataObject, "Table", m.cTable)
ADDPROPERTY(oDataObject, "Alias", m.cAlias)
ADDPROPERTY(oDataObject, "Count")

* Add the object to the collection, using the alias as the key
This.oData.Add(oDataObject, m.cAlias)

RETURN 
ENDPROC

PROCEDURE OpenData

LOCAL oTableInfo, lReturn

lReturn = .T.
FOR EACH oTableInfo IN This.oData FOXOBJECT 
	TRY
		cAlias = oTableInfo.Alias
		USE (oTableInfo.Table) ALIAS (m.cAlias) IN 0
		oTableInfo.Count = RECCOUNT(m.cAlias)
		
	CATCH
		MESSAGEBOX("Cannot open table: " + oTableInfo.Table)
		lReturn = .F.
	ENDTRY
ENDFOR

RETURN lReturn
ENDPROC

PROCEDURE CloseData
* Close the data tables

LOCAL oTableInfo

FOR EACH oTableInfo IN This.oData FOXOBJECT 
	cAlias = oTableInfo.Alias
	TRY 
		USE IN (m.cAlias)
		This.oData.Remove(oTableInfo)
	CATCH
	ENDTRY
ENDFOR

RETURN
ENDPROC

PROCEDURE GetDataCount(cAlias)
* Return the number of records in the specified data table

LOCAL oTable, nCount

TRY
	oTable = This.oData(cAlias)
	nCount = oTable.Count
CATCH
	nCount = 0
ENDTRY

RETURN nCount
ENDPROC

PROCEDURE SetProbabilities
* Abstract
RETURN
ENDPROC

PROCEDURE SetMethods
* Set up the methods to call for the record
* Abstract

RETURN
ENDPROC

PROCEDURE AddMethod(cMethodName, cGroup)
* Add a method to the methods collection

LOCAL oMethod

* Make sure collection exists
IF VARTYPE(This.oMethods) <> "O"
	This.oMethods = CREATEOBJECT("Collection")
ENDIF

* Create the method object
oMethod = CREATEOBJECT("Empty")
ADDPROPERTY(oMethod, "Name", m.cMethodName)
ADDPROPERTY(oMethod, "Group", m.cGroup)

This.oMethods.Add(oMethod, m.cGroup + "|" + m.cMethodName )

RETURN
ENDPROC

PROCEDURE GenerateRecord(cRecordType)

LOCAL oMethod, cMethod

This.oRecord = CREATEOBJECT("Empty")

FOR EACH oMethod IN This.oMethods FOXOBJECT 
	IF oMethod.Group == m.cRecordType
		cMethod = "This." + oMethod.Name
		&cMethod
	ENDIF 
ENDFOR 

RETURN This.oRecord
ENDPROC

PROCEDURE About
*==============================================================================
* Class:				MAKERECORD
* Based On: 			Custom
* Purpose:				Abstract class for creating a test data record.
* Author:				Tamar Granor
* Copyright:			(c) 2005 Tamar E. Granor, Ph.D.
* Last revision:		08/09/05
* Include file:			None

* Changes in "Based On" class properties:
*	None
*
* Changes in "Based On" class methods:
*	Destroy - clean-up
*	Init - set-up
*
* Custom public properties added:
*	oData - collection of tables containing raw data
*	oMethods - collection of methods to call to create record
*	oRecord - object reference to generated record 
*
* Custom protected properties added:
*	None
*
* Custom public methods added:
*	About - this documentation method.
*	AddData - adds an item to the oData collection.
*	AddMethod - adds an item to the oMethods collection.
*	CloseData - closes table specified by oData collection.
*	GenerateRecord - calls the specified methods to create a new record. Abstract.
*	GetDataCount - returns the number of records in the specified table.
*	OpenData - opens tables specified by oData collection.
*	SetData - sets up oData collection. Abstract.
*	SetMethods - sets up oMethods collection. Abstract.
*	SetProbabilities - sets up the probabilities that various data items are present. Abstract.
*
* Custom protected methods added:
*	RandInt - returns a random integer between specified limits.
*	RandLetter - returns a random unpper-case letter.
*	
*==============================================================================


ENDDEFINE

